%% LPV Model Order Reduction By Parameter-Varying Oblique Projection
%  paper submitted to the IEEE Transactions on Control Systems Technology
%  authors Julian Theis, Peter Seiler, Herbert Werner
%
% X56 longitudinal dynamics example
% Code by Julian Theis, Apr 04 2016 
%
% Thanks to Brian Danowsky, Chase Schulze, and Thuan Lieu for providing the
% X56A model
%
% This Matlab Code requires the LPVTools toolbox.
% The LPVTools are openly available from
% http://www.aem.umn.edu/~SeilerControl/software.shtml
%%
clearvars; close all; clc
addpath('./auxil')
load X56long

bopt = bodeoptions; bopt.PhaseMatching='on'; bopt.PhaseMatchingFreq=1;

figure(1); bode(sys,{0.1 1000},bopt)
varypzmap(sys.Data); xlim([-100,10]); ylim([-10,100]); sgrid

%%
opt.method = 'freq';
opt.FreqWeightIn = rolloff(100,5);
opt.FreqWeightOut = rolloff(100,5);
[ SYSR, V, W, INFO ] = lpvprojmr(sys, 12,opt);

figure(1);clf;
bodemag(sys,SYSR,{1,1000},bopt);legend('original','reduced')

%%
SYSRint = lpvinterp(SYSR,'V',ceil(sys.Parameter.V.Range(1)):0.25:floor(sys.Parameter.V.Range(2))); title('original interpolated')
sysrint = lpvinterp(sys,'V',ceil(sys.Parameter.V.Range(1)):0.25:floor(sys.Parameter.V.Range(2))); title('reduced interpolated')

%% Pole-Maps
varypzmap(sys.Data); xlim([-50,5]); ylim([-10,100]);sgrid(0.1:0.1:0.9,10:10:100)
title(['original (' int2str(size(sys.a,1)) ' states)'])
varypzmap(sysrint.Data); xlim([-50,5]); ylim([-10,100]);sgrid(0.1:0.1:0.9,10:10:100)
title(['original interpolated (' int2str(size(sys.a,1)) ' states)'])

varypzmap(SYSR.Data); xlim([-50,5]); ylim([-10,100]);sgrid(0.1:0.1:0.9,10:10:100)
title(['reduced (' int2str(size(SYSR.a,1)) ' states)'])
varypzmap(SYSRint.Data); xlim([-50,5]); ylim([-10,100]);sgrid(0.1:0.1:0.9,10:10:100)
title(['reduced interpolated (' int2str(size(SYSR.a,1)) ' states)'])

%% PointWise Bode Plots
figure(11);
bode(sys,SYSR,{0.1,100},bopt); 
legend(['original (' int2str(size(sys.a,1)) ' states)'],...
       ['reduced (' int2str(size(SYSR.a,1)) ' states)'])
figure(12);
bode(sysrint,SYSRint,{0.1,100},bopt); 
legend(['original interpolated (' int2str(size(sysrint.a,1)) ' states)'],...
       ['reduced interpolated (' int2str(size(SYSRint.a,1)) ' states)'])

%% PointWise Step Response
figure(13)
step(sys,SYSR,2); legend(['original (' int2str(size(sys.a,1)) ' states)'],...
                          ['reduced (' int2str(size(SYSR.a,1)) ' states)'])
figure(14)
step(sysrint,SYSRint,2); legend(['original interpolated (' int2str(size(sys.a,1)) ' states)'],...
                                ['reduced interpolated (' int2str(size(SYSR.a,1)) ' states)'])

%% LPV Step Response
PTRAJ.time  =   0:0.002:2;
Apara       =   0.5*(sys.Parameter.V.Range(2) - sys.Parameter.V.Range(1)); % amplitude of parameter trajectory
Wpara       =   pi; % frequency of parameter trajectory
PTRAJ.V     =   sys.Parameter.V.Range(1) + Apara + 0.95*Apara*cos(Wpara*PTRAJ.time-pi/2);
figure
plot(PTRAJ.time,PTRAJ.V); title('Parameter Trajectory')

[Yo,To] = lpvstep(sys,PTRAJ);
[Yr,Tr] = lpvstep(SYSR,PTRAJ);

figure(15)
subplot(3,2,1)
plot(To,Yo(:,1,1),Tr,Yr(:,1,1));
subplot(3,2,2)
plot(To,Yo(:,1,2),Tr,Yr(:,1,2));
subplot(3,2,3)
plot(To,Yo(:,2,1),Tr,Yr(:,2,1));
subplot(3,2,4)
plot(To,Yo(:,2,2),Tr,Yr(:,2,2));
subplot(3,2,5)
plot(To,Yo(:,3,1),Tr,Yr(:,3,1));
subplot(3,2,6)
plot(To,Yo(:,3,2),Tr,Yr(:,3,2));

 legend(['original (' int2str(size(sys.a,1)) ' states)'],...
        ['reduced (' int2str(size(SYSR.a,1)) ' states)'])
